============
Contributing
============

Contributions are welcome, and they are greatly appreciated!  Every little bit
helps, and credit will always be given.

You can contribute in many ways:

Types of Contributions
----------------------

Report Bugs
~~~~~~~~~~~

Report bugs at https://gitlab.com/carmenbbakker/pyparadox/issues.

If you are reporting a bug, please include:

* Your operating system name and version.
* Any details about your local setup that might be helpful in troubleshooting.
* Detailed steps to reproduce the bug.

Fix Issues
~~~~~~~~~~

Look through the GitLab issues for bugs.  Any open issues are open for anybody
to contribute to.

Write Documentation
~~~~~~~~~~~~~~~~~~~

PyParadox could always use more documentation, primarily in the official docs
and in the docstrings.

Submit Feedback
~~~~~~~~~~~~~~~

The best way to send feedback is to file an issue at
https://gitlab.com/carmenbbakker/pyparadox/issues.

If you are proposing a feature:

* Explain in detail how it would work.
* Keep the scope as narrow as possible, to make it easier to implement.
* Remember that this is a volunteer-driven project, and that contributions
  are welcome :)

Get Started!
------------

Ready to contribute?  Here's how to set up `pyparadox` for local development.

1. Fork the `pyparadox` repo on GitLab.
2. Clone your fork locally::

    git clone git@gitlab.com:your_name_here/pyparadox.git
    cd pyparadox/

3. Install PyParadox and all its development dependencies into a virtualenv.
   The below commands assume you have virtualenvwrapper installed.  PyParadox
   will be installed in-place, and the development dependencies will be
   installed into your virtualenv's `site-packages`.  Also link your local
   installation of PyQt5 into your virtualenv::

    mkvirtualenv pyparadox
    pip install -r requirements/dev.txt
    make link-pyqt

4. Create a branch for local development::

    git checkout -b name-of-your-bugfix-or-feature

   Now you can make your changes locally.

5. When you're done making changes, verify that your changes still pass all
   tests::

    tox

6. Commit your changes and push your branch to GitLab::

    git add your_files
    git commit -m "Your detailed description of your changes."
    git push origin name-of-your-bugfix-or-feature

7. Submit a pull request through the GitLab website.

Pull Request Guidelines
-----------------------

Before you submit a pull request, check that it meets these guidelines:

1. The pull request should include tests.
2. If the pull request adds functionality, the docs should be updated.  Put
   your new functionality into a function with a docstring, and add the feature
   to the list in README.rst.
